==> xcsoar-7.12.tar.xz: xcsoar-7.12/src/Math/ConvexFilter.hpp <==
/*
Copyright_License {

  XCSoar Glide Computer - http://www.xcsoar.org/
  Copyright (C) 2000-2021 The XCSoar Project
  A detailed list of copyright holders can be found in the file "AUTHORS".

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
}
*/

/**
 * Implements a filter that maintains a convex shape
 */

#ifndef _CONVEXFILTER_H
#define _CONVEXFILTER_H

#include "Math/LeastSquares.hpp"

#include <type_traits>

class ConvexFilter: public LeastSquares
{
public:

  /**
   * Add a new data point to the values and convex solution
   *
   * @param x x-Value of the new data point
   * @param y y-Value of the new data point
   */
  void UpdateConvexPositive(double x, double y) noexcept {
    UpdateConvex(x, y, 1);
  }
  void UpdateConvexNegative(double x, double y) noexcept {
    UpdateConvex(x, y, -1);
  }

  double GetLastY() const noexcept;

private:
  void UpdateConvex(double x, double y, int csign) noexcept;
};

static_assert(std::is_trivial<ConvexFilter>::value, "type is not trivial");

#endif // _CONVEXFILTER_H

==> xcsoar-7.20.tar.xz: xcsoar-7.20/src/Math/ConvexFilter.hpp <==
/*
Copyright_License {

  XCSoar Glide Computer - http://www.xcsoar.org/
  Copyright (C) 2000-2021 The XCSoar Project
  A detailed list of copyright holders can be found in the file "AUTHORS".

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
}
*/

/**
 * Implements a filter that maintains a convex shape
 */

#ifndef _CONVEXFILTER_H
#define _CONVEXFILTER_H

#include "Math/LeastSquares.hpp"

#include <cassert>
#include <type_traits>

class ConvexFilter: public LeastSquares
{
public:

  /**
   * Add a new data point to the values and convex solution
   *
   * @param x x-Value of the new data point
   * @param y y-Value of the new data point
   */
  void UpdateConvexPositive(double x, double y) noexcept {
    UpdateConvex(x, y, 1);
  }
  void UpdateConvexNegative(double x, double y) noexcept {
    UpdateConvex(x, y, -1);
  }

  double GetLastY() const noexcept {
    assert(!IsEmpty());

    return GetSlots()[GetCount() - 1].y;
  }

private:
  void UpdateConvex(double x, double y, int csign) noexcept;
};

static_assert(std::is_trivial<ConvexFilter>::value, "type is not trivial");

#endif // _CONVEXFILTER_H
